;;;;; Converted from the "Objects" Processing example at:
;;;;; "http://www.processing.org/learning/examples/objects.html"
;;;;; (C)2006 Luke J Crook

(in-package #:sdl-examples)

(defvar *objects-width* 200)
(defvar *objects-height* 200)

(defclass m-rect ()
  ((w :accessor w :initform 0 :initarg :w)
   (xpos :accessor xpos :initform 0 :initarg :xpos)
   (h :accessor h :initform 0 :initarg :h)
   (ypos :accessor ypos :initform 0 :initarg :ypos)
   (d :accessor d :initform 0 :initarg :d)
   (tt :accessor tt :initform 0 :initarg :tt)))

(defvar *r1* (make-instance 'm-rect :w 1 :xpos 134.0 :h 0.532 :ypos (* 0.083 *objects-height*) :d 10.0 :tt 60.0))
(defvar *r2* (make-instance 'm-rect :w 2 :xpos 44.0 :h 0.166 :ypos (* 0.332 *objects-height*) :d 5.0 :tt 50.0))
(defvar *r3* (make-instance 'm-rect :w 2 :xpos 58.0 :h 0.332 :ypos (* 0.4482 *objects-height*) :d 10.0 :tt 35.0))
(defvar *r4* (make-instance 'm-rect :w 1 :xpos 120.0 :h 0.0498 :ypos (* 0.913 *objects-height*) :d 15.0 :tt 60.0))

(defgeneric move-to-y (rect posy damping))
(defmethod move-to-y ((rect m-rect) posy damping)
  (let ((dif (- (ypos rect) posy)))
    (if (> (abs dif) 1)
	(decf (ypos rect) (/ dif damping)))))

(defgeneric move-to-x (rect posx damping))
(defmethod move-to-x ((rect m-rect) posx damping)
  (let ((dif (- (xpos rect) posx)))
    (if (> (abs dif) 1)
	(decf (xpos rect) (/ dif damping)))))

(defgeneric display (rect &key surface))
(defmethod display ((rect m-rect) &key (surface sdl:*default-display*))
  (sdl:with-color (col (sdl:color :r 255 :g 255 :b 255))
    (dotimes (i (tt rect))
      (sdl:draw-box (sdl:rectangle :x (sdl-base::clamp-to-sshort (+ (xpos rect)
								    (* i (+ (d rect)
									    (w rect)))))
				   :y (sdl-base::clamp-to-sshort (ypos rect))
				   :w (sdl-base::clamp-to-sshort (w rect))
				   :h (sdl-base::clamp-to-sshort (* (h rect)
								    (sdl:height surface))))
		    :surface surface))))

(defun objects ()
  (let ((mouse-x 0) (mouse-y 0))
    (sdl:with-init ()
      (sdl:window *objects-width* *objects-height* :title-caption "Objects, from Processing.org")
      (setf (sdl:frame-rate) 60)
      (sdl:clear-display (sdl:color))
      (sdl:with-events ()
	(:quit-event () t)
	(:video-expose-event () (sdl:update-display))
	(:mouse-motion-event (:x x :y y)
			     (setf mouse-x x
				   mouse-y y))
	(:idle ()
	       (sdl:clear-display (sdl:color))
	       (display *r1*)
	       (display *r2*)
	       (display *r3*)
	       (display *r4*)
	       
	       (move-to-x *r1* (- mouse-x (/ *objects-width* 2)) 30)
	       (move-to-x *r2* (mod (+ mouse-x (* *objects-width* 0.05)) *objects-width*) 20)
	       (move-to-x *r3* (/ mouse-x 4) 40)
	       (move-to-x *r4* (- mouse-x (/ *objects-width* 2)) 50)
	       
	       (move-to-y *r1* (+ mouse-y (* *objects-height* 0.1)) 30)
	       (move-to-y *r2* (+ mouse-y (* *objects-height* 0.025)) 20)
	       (move-to-y *r3* (- mouse-y (* *objects-height* 0.025)) 40)
	       (move-to-y *r4* (- *objects-height* mouse-y) 50)
		 
	       (sdl:update-display))))))

